@@ -71,8 +71,8 @@ void help()
 int main(int args, char** argv)
 {
   cout << "******************************************************************" << endl;
-  cout << "*              Bio++ Alignment Score, version 0.1                *" << endl;
-  cout << "* Author: J. Dutheil                        Last Modif. 15/12/11 *" << endl;
+  cout << "*              Bio++ Alignment Score, version 2.2.0              *" << endl;
+  cout << "* Author: J. Dutheil                        Last Modif. 25/09/14 *" << endl;
   cout << "******************************************************************" << endl;
   cout << endl;
 
@@ -139,7 +139,7 @@ int main(int args, char** argv)
       string phaseOpt = ApplicationTools::getStringParameter("score.phase", bppalnscore.getParams(), "1");
       if (TextTools::isDecimalInteger(phaseOpt))
       {
-        phase = TextTools::toInt(phaseOpt);
+        phase = TextTools::to<size_t>(phaseOpt);
         if (phase == 0)
           throw Exception("ERROR: positions are 1-based.");
         phase--;
