@@ -58,6 +58,8 @@ using namespace std;
 #include <Bpp/Seq/Container/SiteContainerTools.h>
 #include <Bpp/Seq/SiteTools.h>
 #include <Bpp/Seq/App/SequenceApplicationTools.h>
+#include <Bpp/Text/KeyvalTools.h>
+
 
 // From PhylLib:
 #include <Bpp/Phyl/Tree.h>
@@ -84,9 +86,9 @@ void help()
 int main(int args, char ** argv)
 {
   cout << "******************************************************************" << endl;
-  cout << "*              Bio++ Distance Methods, version 0.3.0             *" << endl;
+  cout << "*              Bio++ Distance Methods, version 2.2.0             *" << endl;
   cout << "* Author: J. Dutheil                        Created     05/05/07 *" << endl;
-  cout << "*                                           Last Modif. 08/08/09 *" << endl;
+  cout << "*                                           Last Modif. 25/09/14 *" << endl;
   cout << "******************************************************************" << endl;
   cout << endl;
 
@@ -102,6 +104,14 @@ int main(int args, char ** argv)
   bppdist.startTimer();
 
   Alphabet* alphabet = SequenceApplicationTools::getAlphabet(bppdist.getParams(), "", false);
+  auto_ptr<GeneticCode> gCode;
+  CodonAlphabet* codonAlphabet = dynamic_cast<CodonAlphabet*>(alphabet);
+  if (codonAlphabet) {
+    string codeDesc = ApplicationTools::getStringParameter("genetic_code", bppdist.getParams(), "Standard", "", true, true);
+    ApplicationTools::displayResult("Genetic Code", codeDesc);
+
+    gCode.reset(SequenceApplicationTools::getGeneticCode(codonAlphabet->getNucleicAlphabet(), codeDesc));
+  }
 
   VectorSiteContainer* allSites = SequenceApplicationTools::getSiteContainer(alphabet, bppdist.getParams());
   
@@ -111,7 +121,7 @@ int main(int args, char ** argv)
   ApplicationTools::displayResult("Number of sequences", TextTools::toString(sites->getNumberOfSequences()));
   ApplicationTools::displayResult("Number of sites", TextTools::toString(sites->getNumberOfSites()));
   
-  SubstitutionModel* model = PhylogeneticsApplicationTools::getSubstitutionModel(alphabet, sites, bppdist.getParams());
+  SubstitutionModel* model = PhylogeneticsApplicationTools::getSubstitutionModel(alphabet, gCode.get(), sites, bppdist.getParams());
   
 	DiscreteDistribution* rDist = 0;
   if (model->getNumberOfStates() > model->getAlphabet()->getSize())
@@ -226,11 +236,32 @@ int main(int args, char ** argv)
   if (matrixPath != "none")
   {
     ApplicationTools::displayResult("Output matrix file", matrixPath);
-    ODistanceMatrix* odm = IODistanceMatrixFactory().createWriter(IODistanceMatrixFactory::PHYLIP_FORMAT);
+    string matrixFormat = ApplicationTools::getAFilePath("output.matrix.format", bppdist.getParams(), false, false, "", false);
+    string format = "";
+    bool extended = false;
+    std::map<std::string, std::string> unparsedArguments_;
+    KeyvalTools::parseProcedure(matrixFormat, format, unparsedArguments_);
+    if (unparsedArguments_.find("type") != unparsedArguments_.end())
+    {
+      if (unparsedArguments_["type"] == "extended")
+      {
+        extended = true;
+      }     
+      else if (unparsedArguments_["type"] == "classic")
+        extended = false;
+      else
+        ApplicationTools::displayWarning("Argument '" +
+                                         unparsedArguments_["type"] + "' for parameter 'Phylip#type' is unknown. " +
+                                         "Default used instead: not extended.");
+    }    
+    else
+      ApplicationTools::displayWarning("Argument 'Phylip#type' not found. Default used instead: not extended.");
+    
+
+    ODistanceMatrix* odm = IODistanceMatrixFactory().createWriter(IODistanceMatrixFactory::PHYLIP_FORMAT, extended);
     odm->write(*distEstimation.getMatrix(), matrixPath, true);
     delete odm;
-  }
-
+   }
   PhylogeneticsApplicationTools::writeTree(*tree, bppdist.getParams());
   
   //Output some parameters:
@@ -331,9 +362,9 @@ int main(int args, char ** argv)
   delete distMethod;
   delete tree;
 
-  bppdist.done();
-
-  }
+  bppdist.done();}
+  
+      
   catch(exception & e)
   {
     cout << e.what() << endl;
