@@ -5,7 +5,7 @@
 //
 
 /*
-   Copyright or © or Copr. CNRS
+   Copyright or © or Copr. Bio++ Development Team
 
    This software is a computer program whose purpose is to estimate
    phylogenies and evolutionary parameters from a dataset according to
@@ -91,7 +91,8 @@ int main(int args, char** argv)
 {
   cout << "******************************************************************" << endl;
   cout << "*     Bio++ Computation of site likelihoods inside mixed models  *" << endl;
-  cout << "* Author: L. Guéguen                        Created on: 12/11/12 *" << endl;
+  cout << "*                        Version 2.2.0.                          *" << endl;
+  cout << "* Author: L. Guéguen                       Last Modif.: 25/09/14 *" << endl;
   cout << "******************************************************************" << endl;
   cout << endl;
 
@@ -107,6 +108,14 @@ int main(int args, char** argv)
     bppmixedlikelihoods.startTimer();
 
     Alphabet* alphabet = SequenceApplicationTools::getAlphabet(bppmixedlikelihoods.getParams(), "", false);
+    auto_ptr<GeneticCode> gCode;
+    CodonAlphabet* codonAlphabet = dynamic_cast<CodonAlphabet*>(alphabet);
+    if (codonAlphabet) {
+      string codeDesc = ApplicationTools::getStringParameter("genetic_code", bppmixedlikelihoods.getParams(), "Standard", "", true, true);
+      ApplicationTools::displayResult("Genetic Code", codeDesc);
+      
+      gCode.reset(SequenceApplicationTools::getGeneticCode(codonAlphabet->getNucleicAlphabet(), codeDesc));
+    }
 
     // get the data
 
@@ -127,13 +136,13 @@ int main(int args, char** argv)
     string nhOpt = ApplicationTools::getStringParameter("nonhomogeneous", bppmixedlikelihoods.getParams(), "no", "", true, false);
     ApplicationTools::displayResult("Heterogeneous model", nhOpt);
 
-    MixedSubstitutionModel* model    = 0;
+    MixedSubstitutionModel* model       = 0;
     MixedSubstitutionModelSet* modelSet = 0;
-    DiscreteDistribution* rDist    = 0;
+    DiscreteDistribution* rDist         = 0;
 
     if (nhOpt == "no")
     {
-      model = dynamic_cast<MixedSubstitutionModel*>(PhylogeneticsApplicationTools::getSubstitutionModel(alphabet, sites, bppmixedlikelihoods.getParams()));
+      model = dynamic_cast<MixedSubstitutionModel*>(PhylogeneticsApplicationTools::getSubstitutionModel(alphabet, gCode.get(), sites, bppmixedlikelihoods.getParams()));
       if (model == 0)
       {
         cout << "Model is not a Mixed model" << endl;
@@ -154,7 +163,7 @@ int main(int args, char** argv)
     }
     else if (nhOpt == "one_per_branch")
     {
-      model = dynamic_cast<MixedSubstitutionModel*>(PhylogeneticsApplicationTools::getSubstitutionModel(alphabet, sites, bppmixedlikelihoods.getParams()));
+      model = dynamic_cast<MixedSubstitutionModel*>(PhylogeneticsApplicationTools::getSubstitutionModel(alphabet, gCode.get(), sites, bppmixedlikelihoods.getParams()));
       if (model == 0)
       {
         cout << "Model is not a Mixed model" << endl;
@@ -179,7 +188,7 @@ int main(int args, char** argv)
         rateFreqs = vector<double>(n, 1. / (double)n); // Equal rates assumed for now, may be changed later (actually, in the most general case,
         // we should assume a rate distribution for the root also!!!
       }
-      FrequenciesSet* rootFreqs = PhylogeneticsApplicationTools::getRootFrequenciesSet(alphabet, sites, bppmixedlikelihoods.getParams(), rateFreqs);
+      FrequenciesSet* rootFreqs = PhylogeneticsApplicationTools::getRootFrequenciesSet(alphabet, gCode.get(), sites, bppmixedlikelihoods.getParams(), rateFreqs);
       vector<string> globalParameters = ApplicationTools::getVectorParameter<string>("nonhomogeneous_one_per_branch.shared_parameters", bppmixedlikelihoods.getParams(), ',', "");
       modelSet = dynamic_cast<MixedSubstitutionModelSet*>(SubstitutionModelSetTools::createNonHomogeneousModelSet(model, rootFreqs, tree, globalParameters));
       model = 0;
@@ -187,7 +196,7 @@ int main(int args, char** argv)
     }
     else if (nhOpt == "general")
     {
-      modelSet = dynamic_cast<MixedSubstitutionModelSet*>(PhylogeneticsApplicationTools::getSubstitutionModelSet(alphabet, sites, bppmixedlikelihoods.getParams()));
+      modelSet = dynamic_cast<MixedSubstitutionModelSet*>(PhylogeneticsApplicationTools::getSubstitutionModelSet(alphabet, gCode.get(), sites, bppmixedlikelihoods.getParams()));
       if (modelSet == 0)
       {
         cout << "Missing a Mixed model" << endl;
@@ -283,14 +292,19 @@ int main(int args, char** argv)
 
     bool fromBiblio=false;
     
-    //this is an uglly fix because getMixedModel is private... can't we use clone instead or const everywhere?
     const AbstractBiblioMixedSubstitutionModel* ptmp = dynamic_cast<const AbstractBiblioMixedSubstitutionModel*>(p0);
     if (ptmp) {
       p0 = ptmp->getMixedModel().clone();
+      if (nhOpt == "no")
+        model = p0;
+      else {
+        modelSet->replaceModel(nummodel-1, p0);
+        modelSet->isFullySetUpFor(*tree);
+      }
       fromBiblio=true;
     }
 
-
+    //////////////////////////////////////////////////
     // Case of a MixtureOfSubstitutionModels
 
     MixtureOfSubstitutionModels* pMSM = dynamic_cast<MixtureOfSubstitutionModels*>(p0);
@@ -350,29 +364,38 @@ int main(int args, char** argv)
       DataTable::write(*rates, out, "\t");
     }
 
+    //////////////////////////////////////////////////
     // Case of a MixtureOfASubstitutionModel
 
     else
     {
-      if (fromBiblio)
-        {
-          ApplicationTools::displayError("!!! Not available for models parametrized upon bibliography.");
-          ApplicationTools::displayError("!!! Please convert into MixedModel declaration.");
-          exit(-1);
-        }
-      
       MixtureOfASubstitutionModel* pMSM2 = dynamic_cast<MixtureOfASubstitutionModel*>(p0);
       if (pMSM2 != NULL)
       {
+        size_t nummod = pMSM2->getNumberOfModels();
+        if (fromBiblio && (parname == ""))
+        {
+          ParameterList pl=pMSM2->getParameters();
+
+          for (size_t i2 = 0; i2 < pl.size(); i2++)
+          {
+            string pl2n = pl[i2].getName();
+            string par2 = pl2n.substr(0,pl2n.find("_")) + "_1";
+            Vint vnmod = pMSM2->getSubmodelNumbers(par2);
+            if (vnmod.size() == 1) {
+              parname=pl2n.substr(0,pl2n.find("_"));
+              break;
+            }
+          }
+        }
+
         if (parname == "")
         {
           ApplicationTools::displayError("Argument likelihoods.parameter_name is required.");
           exit(-1);
         }
 
-        size_t nummod = pMSM2->getNumberOfModels();
-
-        vector<vector<int> > vvnmod;
+        vector< Vint > vvnmod;
         size_t i2 = 0;
         while (i2 < nummod)
         {
@@ -394,9 +417,9 @@ int main(int args, char** argv)
         for (size_t i = 0; i < nbcl; i++)
         {
           vector<double> vprob2;
-          for (unsigned int j = 0; j < vvnmod[i].size(); j++)
+          for (size_t j = 0; j < vvnmod[i].size(); j++)
           {
-            vprob2.push_back(vprob[vvnmod[i][j]]);
+            vprob2.push_back(vprob[static_cast<size_t>(vvnmod[i][j])]);
           }
 
           vvprob.push_back(vprob2);
@@ -408,18 +431,15 @@ int main(int args, char** argv)
 
         Vdouble dval;
         for (unsigned int i = 0; i < nbcl; i++)
-          {
-            SubstitutionModel* pSM = pMSM2->getNModel(vvnmod[i][0]);
-            double valPar = pSM->getParameterValue(pSM->getParameterNameWithoutNamespace(parname));
-            dval.push_back(valPar);
-            colNames.push_back("Ll_" + parname + "=" + TextTools::toString(valPar));
-          }
+        {
+          SubstitutionModel* pSM = pMSM2->getNModel(static_cast<size_t>(vvnmod[i][0]));
+          double valPar = pSM->getParameterValue(pSM->getParameterNameWithoutNamespace(parname));
+          dval.push_back(valPar);
+          colNames.push_back("Ll_" + parname + "=" + TextTools::toString(valPar));
+        }
         for (unsigned int i = 0; i < nbcl; i++)
-          {
-            SubstitutionModel* pSM = pMSM2->getNModel(vvnmod[i][0]);
-            double valPar = pSM->getParameterValue(pSM->getParameterNameWithoutNamespace(parname));
-            colNames.push_back("Pr_" + parname + "=" + TextTools::toString(valPar));
-          }
+          colNames.push_back("Pr_" + parname + "=" + TextTools::toString(dval[i]));
+
         colNames.push_back("mean");
 
         DataTable* rates = new DataTable(nSites, colNames.size());
@@ -434,14 +454,18 @@ int main(int args, char** argv)
 
         VVdouble vvd;
 
-        for (unsigned int i = 0; i < nbcl; i++)
+          
+        vector<double> vRates = pMSM2->getVRates();
+
+        for (size_t i = 0; i < nbcl; ++i)
         {
           string par2 = parname + "_" + TextTools::toString(i + 1);
-          for (unsigned int j = 0; j < nummod; j++)
+          
+          for (unsigned int j = 0; j < nummod; ++j)
             pMSM2->setNProbability(j, 0);
 
-          for (unsigned int j = 0; j < vvprob[i].size(); j++)
-            pMSM2->setNProbability(vvnmod[i][j], vvprob[i][j] / vsprob[i]);
+          for (size_t j = 0; j < vvprob[i].size(); ++j)
+            pMSM2->setNProbability(static_cast<size_t>(vvnmod[i][j]), vvprob[i][j] / vsprob[i]);
 
           if (tl)
             delete tl;
@@ -461,7 +485,7 @@ int main(int args, char** argv)
           vvd.push_back(vd);
 
           ApplicationTools::displayMessage("\n");
-          ApplicationTools::displayMessage("Parameter " + par2 + ":");
+          ApplicationTools::displayMessage("Parameter " + par2 + "=" + TextTools::toString(dval[i]) + " with rate=" + TextTools::toString(vRates[i]));
 
           ApplicationTools::displayResult("Log likelihood", TextTools::toString(tl->getValue(), 15));
           ApplicationTools::displayResult("Probability", TextTools::toString(vsprob[i], 15));
