@@ -5,7 +5,7 @@
 //
 
 /*
-Copyright or © or Copr. CNRS
+Copyright or © or Copr. Bio++ Development Team
 
 This software is a computer program whose purpose is to estimate
 phylogenies and evolutionary parameters from a dataset according to
@@ -78,9 +78,9 @@ int main(int args, char ** argv)
 {
   
   cout << "******************************************************************" << endl;
-  cout << "*                  Bio++ ReRoot, version 0.1.3                   *" << endl;
+  cout << "*                  Bio++ ReRoot, version 2.2.0                   *" << endl;
   cout << "* Author: C. Scornavacca                    Created     15/01/08 *" << endl;
-  cout << "*                                           Last Modif. 08/08/09 *" << endl;
+  cout << "*                                           Last Modif. 25/09/14 *" << endl;
   cout << "******************************************************************" << endl;
   cout << endl;
 
@@ -261,10 +261,10 @@ int main(int args, char ** argv)
                   {
                     sonUpper = (tree->getRootNode())->getSon(0);
                   }
-                  int ident = TreeTools::getMaxId(* low, low->getRootId());
+                  int ident = TreeTools::getMaxId(*low, low->getRootId());
                   vector <Node *> nodesTemp= TreeTemplateTools::getNodes( * sonUpper);
-                  for(unsigned int F = 0; F < nodesTemp.size(); F++)
-                    ( * nodesTemp[F]).setId(ident + F + 1);
+                  for(size_t F = 0; F < nodesTemp.size(); F++)
+                    nodesTemp[F]->setId(ident + static_cast<int>(F + 1));
                   low->getRootNode()->addSon(sonUpper);
                   tree = low;
                 }
@@ -306,9 +306,7 @@ int main(int args, char ** argv)
   ApplicationTools::displayTaskDone();
      
   //Write rooted trees:  
-
-     
-  for (unsigned int i = 0; i < trees.size(); i++) delete trees[i];
+  for (size_t i = 0; i < trees.size(); i++) delete trees[i];
     
   bppreroot.done();
   }
