@@ -49,8 +49,9 @@ using namespace std;
 #include <Bpp/Io/FileTools.h>
 #include <Bpp/Text/KeyvalTools.h>
 
-// From SeqLib:
+// From bpp-seq:
 #include <Bpp/Seq/SiteTools.h>
+#include <Bpp/Seq/CodonSiteTools.h>
 #include <Bpp/Seq/Alphabet/Alphabet.h>
 #include <Bpp/Seq/Alphabet/AlphabetTools.h>
 #include <Bpp/Seq/Container/VectorSiteContainer.h>
@@ -60,7 +61,7 @@ using namespace std;
 #include <Bpp/Seq/SequenceTools.h>
 #include <Bpp/Seq/GeneticCode.all>
 
-//From PhylLib:
+//From bpp-phyl:
 #include <Bpp/Phyl/Tree.h>
 #include <Bpp/Phyl/App/PhylogeneticsApplicationTools.h>
 
@@ -79,8 +80,8 @@ void help()
 int main(int args, char** argv)
 {
   cout << "******************************************************************" << endl;
-  cout << "*           Bio++ Sequence Manipulator, version 0.6              *" << endl;
-  cout << "* Author: J. Dutheil                        Last Modif. 21/12/11 *" << endl;
+  cout << "*           Bio++ Sequence Manipulator, version 2.2.0.           *" << endl;
+  cout << "* Author: J. Dutheil                        Last Modif. 25/09/14 *" << endl;
   cout << "******************************************************************" << endl;
   cout << endl;
   
@@ -97,6 +98,8 @@ int main(int args, char** argv)
   
   // Get alphabet
   Alphabet* alphabet = SequenceApplicationTools::getAlphabet(bppseqman.getParams(), "", false, true, true);
+  auto_ptr<GeneticCode> gCode;
+  CodonAlphabet* codonAlphabet = dynamic_cast<CodonAlphabet*>(alphabet);
 
   // Get sequences:
   SequenceContainer* tmp = SequenceApplicationTools::getSequenceContainer(alphabet, bppseqman.getParams(), "", true, true);
@@ -106,7 +109,7 @@ int main(int args, char** argv)
   
   // Perform manipulations
   
-  vector<string> actions = ApplicationTools::getVectorParameter<string>("sequence.manip", bppseqman.getParams(), ',', "", "", false, false);
+  vector<string> actions = ApplicationTools::getVectorParameter<string>("sequence.manip", bppseqman.getParams(), ',', "", "", false, 1);
   
   bool aligned = false;
 
@@ -124,7 +127,7 @@ int main(int args, char** argv)
     {
       OrderedSequenceContainer* sc = 0;
       if (aligned) sc = new VectorSiteContainer(sequences->getAlphabet());
-      else         sc = reinterpret_cast<OrderedSequenceContainer*>(new VectorSequenceContainer(sequences->getAlphabet()));
+      else         sc = new VectorSequenceContainer(sequences->getAlphabet());
       for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
       {
         Sequence* seq = SequenceTools::getComplement(sequences->getSequence(i));
@@ -143,7 +146,7 @@ int main(int args, char** argv)
       {
         OrderedSequenceContainer* sc = 0;
         if (aligned) sc = new VectorSiteContainer(&AlphabetTools::RNA_ALPHABET);
-        else         sc = reinterpret_cast<OrderedSequenceContainer*>(new VectorSequenceContainer(&AlphabetTools::RNA_ALPHABET));
+        else         sc = new VectorSequenceContainer(&AlphabetTools::RNA_ALPHABET);
         for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
         {
           Sequence* seq = SequenceTools::transcript(sequences->getSequence(i));
@@ -157,7 +160,7 @@ int main(int args, char** argv)
       {
         OrderedSequenceContainer* sc = 0;
         if (aligned) sc = new VectorSiteContainer(&AlphabetTools::DNA_ALPHABET);
-        else         sc = reinterpret_cast<OrderedSequenceContainer*>(new VectorSequenceContainer(&AlphabetTools::DNA_ALPHABET));
+        else         sc = new VectorSequenceContainer(&AlphabetTools::DNA_ALPHABET);
         for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
         {
           Sequence* seq = SequenceTools::reverseTranscript(sequences->getSequence(i));
@@ -186,7 +189,7 @@ int main(int args, char** argv)
       else throw Exception("Cannot switch alphabet type, alphabet is not of type 'nucleic'.");
       OrderedSequenceContainer* sc = 0;
       if (aligned) sc = new VectorSiteContainer(alpha);
-      else         sc = reinterpret_cast<OrderedSequenceContainer*>(new VectorSequenceContainer(alpha));
+      else         sc = new VectorSequenceContainer(alpha);
       for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
       {
         const Sequence* old = &sequences->getSequence(i);
@@ -204,16 +207,20 @@ int main(int args, char** argv)
     {
       if (!AlphabetTools::isCodonAlphabet(sequences->getAlphabet()))
         throw Exception("Error in translation: alphabet is not of type 'codon'.");
-      GeneticCode* gc = NULL;
-      string gcstr = ApplicationTools::getStringParameter("code", cmdArgs, "Standard");
-      gc = SequenceApplicationTools::getGeneticCode(dynamic_cast<const CodonAlphabet*>(sequences->getAlphabet())->getNucleicAlphabet(), gcstr);
+      if (cmdArgs["code"] != "")
+        throw Exception("ERROR: 'code' argument is deprecated. The genetic code to use for translation is now set by the top-level argument 'genetic_code'.");
+      if (!gCode.get()) {
+        string codeDesc = ApplicationTools::getStringParameter("genetic_code", bppseqman.getParams(), "Standard", "", true, 1);
+        ApplicationTools::displayResult("Genetic Code", codeDesc);
+        gCode.reset(SequenceApplicationTools::getGeneticCode(codonAlphabet->getNucleicAlphabet(), codeDesc));
+      }
 
       OrderedSequenceContainer* sc = 0;
       if (aligned) sc = new VectorSiteContainer(&AlphabetTools::PROTEIN_ALPHABET);
-      else         sc = reinterpret_cast<OrderedSequenceContainer*>(new VectorSequenceContainer(&AlphabetTools::PROTEIN_ALPHABET));
-      for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
+      else         sc = new VectorSequenceContainer(&AlphabetTools::PROTEIN_ALPHABET);
+      for (size_t i = 0; i < sequences->getNumberOfSequences(); ++i)
       {
-        Sequence* seq = gc->translate(sequences->getSequence(i));
+        Sequence* seq = gCode->translate(sequences->getSequence(i));
         sc->addSequence(*seq, false);
         delete seq;
       }
@@ -243,7 +250,7 @@ int main(int args, char** argv)
     {
       OrderedSequenceContainer* sc = 0;
       if (aligned) sc = new VectorSiteContainer(sequences->getAlphabet());
-      else         sc = reinterpret_cast<OrderedSequenceContainer*>(new VectorSequenceContainer(sequences->getAlphabet()));
+      else         sc = new VectorSequenceContainer(sequences->getAlphabet());
       for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
       {
         Sequence* seq = new BasicSequence(sequences->getSequence(i));
@@ -278,24 +285,29 @@ int main(int args, char** argv)
     // +--------------+
     else if (cmdName == "RemoveStops")
     {
+      if (!gCode.get()) {
+        string codeDesc = ApplicationTools::getStringParameter("genetic_code", bppseqman.getParams(), "Standard", "", true, 1);
+        ApplicationTools::displayResult("Genetic Code", codeDesc);
+        gCode.reset(SequenceApplicationTools::getGeneticCode(codonAlphabet->getNucleicAlphabet(), codeDesc));
+      }
       SiteContainer* sites = dynamic_cast<SiteContainer*>(sequences);
       if (!sites)
       {
         VectorSequenceContainer* sc = new VectorSequenceContainer(sequences->getAlphabet());
-        for (unsigned int i = 0; i < sequences->getNumberOfSequences(); ++i)
+        for (size_t i = 0; i < sequences->getNumberOfSequences(); ++i)
         {
           auto_ptr<Sequence> seq(sequences->getSequence(i).clone());
-          SequenceTools::removeStops(*seq);
+          SequenceTools::removeStops(*seq, *gCode);
           sc->addSequence(*seq);
         }
         delete sequences;
         sequences = sc;
       } else {
         VectorSiteContainer* sc = new VectorSiteContainer(sequences->getAlphabet());
-        for (unsigned int i = 0; i < sequences->getNumberOfSequences(); ++i)
+        for (size_t i = 0; i < sequences->getNumberOfSequences(); ++i)
         {
           auto_ptr<Sequence> seq(sequences->getSequence(i).clone());
-          SequenceTools::replaceStopsWithGaps(*seq);
+          SequenceTools::replaceStopsWithGaps(*seq, *gCode);
           sc->addSequence(*seq);
         }
         delete sequences;
@@ -313,10 +325,15 @@ int main(int args, char** argv)
       {
         throw Exception("'RemoveColumnsWithStop' can only be used on alignment. You may consider using the 'CoerceToAlignment' command.");
       }
+      if (!gCode.get()) {
+        string codeDesc = ApplicationTools::getStringParameter("genetic_code", bppseqman.getParams(), "Standard", "", true, 1);
+        ApplicationTools::displayResult("Genetic Code", codeDesc);
+        gCode.reset(SequenceApplicationTools::getGeneticCode(codonAlphabet->getNucleicAlphabet(), codeDesc));
+      }
 
       for (size_t i = sites->getNumberOfSites(); i > 0; i--)
       {
-        if (SiteTools::hasStopCodon(sites->getSite(i-1)))
+        if (CodonSiteTools::hasStop(sites->getSite(i-1), *gCode))
           sites->deleteSite(i - 1);
       }
     }
@@ -326,14 +343,19 @@ int main(int args, char** argv)
     // +---------+
     else if (cmdName == "GetCDS")
     {
+      if (!gCode.get()) {
+        string codeDesc = ApplicationTools::getStringParameter("genetic_code", bppseqman.getParams(), "Standard", "", true, 1);
+        ApplicationTools::displayResult("Genetic Code", codeDesc);
+        gCode.reset(SequenceApplicationTools::getGeneticCode(codonAlphabet->getNucleicAlphabet(), codeDesc));
+      }
       OrderedSequenceContainer* sc = 0;
       if (aligned) sc = new VectorSiteContainer(sequences->getAlphabet());
-      else         sc = reinterpret_cast<OrderedSequenceContainer*>(new VectorSequenceContainer(sequences->getAlphabet()));
-      for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
+      else         sc = new VectorSequenceContainer(sequences->getAlphabet());
+      for (size_t i = 0; i < sequences->getNumberOfSequences(); ++i)
       {
         BasicSequence seq = sequences->getSequence(i);
         size_t len = seq.size();
-        SequenceTools::getCDS(seq, false, true, true, false);
+        SequenceTools::getCDS(seq, *gCode, false, true, true, false);
         if (aligned) {
           for (size_t c = seq.size(); c < len; ++c)
             seq.addElement(seq.getAlphabet()->getGapCharacterCode());
@@ -367,7 +389,7 @@ int main(int args, char** argv)
       }
 
       const Alphabet* alpha = 0;
-      string alphastr = ApplicationTools::getStringParameter("alphabet", cmdArgs, "DNA");
+      string alphastr = ApplicationTools::getStringParameter("alphabet", cmdArgs, "DNA", "", false, 1);
       if (alphastr == "DNA") alpha = &AlphabetTools::DNA_ALPHABET;
       else if (alphastr == "RNA") alpha = &AlphabetTools::RNA_ALPHABET;
       else if (alphastr == "Protein") alpha = &AlphabetTools::PROTEIN_ALPHABET;
@@ -387,7 +409,7 @@ int main(int args, char** argv)
         throw Exception("'KeepComplete' can only be used on alignment. You may consider using the 'CoerceToAlignment' command.");
       }
 
-      string maxGapOption = ApplicationTools::getStringParameter("maxGapAllowed", cmdArgs, "100%");
+      string maxGapOption = ApplicationTools::getStringParameter("maxGapAllowed", cmdArgs, "100%", "", false, 1);
       if (maxGapOption[maxGapOption.size()-1] == '%')
       {
         double gapFreq = TextTools::toDouble(maxGapOption.substr(0, maxGapOption.size()-1)) / 100.;
@@ -417,7 +439,7 @@ int main(int args, char** argv)
     {
       OrderedSequenceContainer* sc = 0;
       if (aligned) sc = new VectorSiteContainer(sequences->getAlphabet());
-      else         sc = reinterpret_cast<OrderedSequenceContainer*>(new VectorSequenceContainer(sequences->getAlphabet()));
+      else         sc = new VectorSequenceContainer(sequences->getAlphabet());
       for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
       {
         const Sequence* old = &sequences->getSequence(i);
@@ -433,7 +455,7 @@ int main(int args, char** argv)
     // +------------------+
     else if (cmdName == "GetCodonPosition")
     {
-      unsigned int pos = ApplicationTools::getParameter<unsigned int>("position", cmdArgs, 3);
+      unsigned int pos = ApplicationTools::getParameter<unsigned int>("position", cmdArgs, 3, "", false, 1);
       OrderedSequenceContainer* sc = dynamic_cast<OrderedSequenceContainer*>(SequenceContainerTools::getCodonPosition(*sequences, pos - 1));
       delete sequences;
       if (aligned) {
@@ -470,11 +492,11 @@ int main(int args, char** argv)
   ApplicationTools::displayBooleanResult("Final sequences are aligned", aligned);
   if (aligned)
   {
-    SequenceApplicationTools::writeAlignmentFile(*dynamic_cast<SiteContainer*>(sequences), bppseqman.getParams(), "", true);
+    SequenceApplicationTools::writeAlignmentFile(*dynamic_cast<SiteContainer*>(sequences), bppseqman.getParams(), "", true, 1);
   }
   else
   {
-    SequenceApplicationTools::writeSequenceFile(*sequences, bppseqman.getParams(), "", true);
+    SequenceApplicationTools::writeSequenceFile(*sequences, bppseqman.getParams(), "", true, 1);
   }
 
   delete alphabet;
